package edu.gatech.cercs.soa.core;


import edu.gatech.cercs.soa.core.fpq.IFlightPricingQuery;
import edu.gatech.cercs.soa.core.fpq.IHeader;
import edu.gatech.cercs.soa.core.fpq.impl.FlightPricingQueryImpl;
import edu.gatech.cercs.soa.core.fpq.impl.HeaderImpl;
import edu.gatech.cercs.soa.core.fpr.IFlightPricingResult;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.core.fpr.impl.FlightPricingResultImpl;
import edu.gatech.cercs.soa.core.fpr.impl.HeaderResultImpl;

/**
 * The FlightPricingFactory hides the construction details of the flight pricing
 * query, result and result header objects.
 * 
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingFactory {

   public static IFlightPricingQuery createQuery() {
      IFlightPricingQuery fpQuery = new FlightPricingQueryImpl();
      fpQuery.getHeader().setQueryId(FlightPricingUtils.createUniqueQueryId());
      return fpQuery;
   }

   public static IFlightPricingResult createResult() {
      return new FlightPricingResultImpl();
   }

   public static IHeader createQueryHeader() {
      return new HeaderImpl();
   }

   public static IHeaderResult createHeaderResult() {
      return new HeaderResultImpl();
   }

   public static IFlightPricingResult createStatusResult(String status,
            String statusDetail) {
      IFlightPricingResult fpr = new FlightPricingResultImpl();
      fpr.getHeader().setStatus(status);
      fpr.getHeader().setStatusDetail(statusDetail);
      return fpr;
   }

   public static IFlightPricingResult createErrorResult(String errorMsg) {
      return createStatusResult(IHeaderResult.STATUS_ERROR, errorMsg);
   }

   public static IFlightPricingResult createQueueResult(String msg) {
      return createStatusResult(IHeaderResult.STATUS_QUEUE, msg);
   }

   public static IFlightPricingResult createAcknowledgeResult(String ackMsg) {
      return createStatusResult(IHeaderResult.STATUS_ACKNOWLEDGE, ackMsg);
   }

}