package edu.gatech.cercs.soa.core.fpq.impl;

import java.io.PrintWriter;
import java.util.Map;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpq.*;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>FlightPricingQueryImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingQueryImpl implements IFlightPricingQuery, XMLable {

    private IHeader hdr;
    private IAirlines als;
    private IFareClasses fcs;
    private IItinerary itny;
    private IPassengers ps;
    
    public FlightPricingQueryImpl() {
        hdr = new HeaderImpl();
    }
    
    public IAirlines getAirlines() {
        if (als == null) {
            als = new AirlinesImpl();
        }
        return als;
    }

    public IFareClasses getFareClasses() {
        if (fcs == null) {
            fcs = new FareClassesImpl();
        }
        return fcs;
    }
    
    public IHeader getHeader() {
        if (hdr == null) {
            hdr = new HeaderImpl();
        }
        return hdr;
    }

    public IItinerary getItinerary() {
        if (itny == null) {
            itny = new ItineraryImpl();
        }
        return itny;
    }
    
    public IPassengers getPassengers() {
        if (ps == null) {
            ps = new PassengersImpl();
        }
        return ps;
    }

    public void setAirlines(IAirlines als) {
        this.als = als;
    }

    public void setFareClasses(IFareClasses fc) {
        this.fcs = fc;
    }
    
    public void setHeader(IHeader hdr) {
        this.hdr = hdr;
    }

    public void setItinerary(IItinerary it) {
        this.itny = it;
    }

    public void setPassengers(IPassengers ps) {
        this.ps = ps;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
            if (name.equalsIgnoreCase("header")) {
                state.put("header", getHeader());
            }
            else if (name.equalsIgnoreCase("itinerary")) {
                state.put("itinerary", new ItineraryImpl());
            }
            else if (name.equalsIgnoreCase("passengers")) {
                state.put("passengers", new PassengersImpl());
            }
            else if (name.equalsIgnoreCase("fareclasses")) {
                state.put("fareclasses", new FareClassesImpl());
            }
            else if (name.equalsIgnoreCase("airlines")) {
                state.put("airlines", new AirlinesImpl());
            }
        }
        else {
            //Closing Tag
            if (name.equalsIgnoreCase("header")) {
                this.hdr = (IHeader)state.remove("header");
            }
            else if (name.equalsIgnoreCase("itinerary")) {
                this.itny = (IItinerary)state.remove("itinerary");
            }
            else if (name.equalsIgnoreCase("passengers")) {
                this.ps = (IPassengers)state.remove("passengers");
            }
            else if (name.equalsIgnoreCase("fareclasses")) {
                this.fcs = (IFareClasses)state.remove("fareclasses");
            }
            else if (name.equalsIgnoreCase("airlines")) {
                this.als = (IAirlines)state.remove("airlines");
            }
        }

        IHeader hdr = (IHeader)state.get("header");
        if (hdr != null) {
            ((XMLable)hdr).handleXMLCallback(name, attrs, isBeginElement, state);
            return;
        }
        
        IItinerary itny = (IItinerary)state.get("itinerary");
        if (itny != null) {
            ((XMLable)itny).handleXMLCallback(name, attrs, isBeginElement, state);
            return;
        }
        
        IPassengers ps = (IPassengers)state.get("passengers");
        if (ps != null) {
            ((XMLable)ps).handleXMLCallback(name, attrs, isBeginElement, state);
            return;
        }
        
        IFareClasses fcs = (IFareClasses)state.get("fareclasses");
        if (fcs != null) {
            ((XMLable)fcs).handleXMLCallback(name, attrs, isBeginElement, state);
            return;
        }
        
        IAirlines als = (IAirlines)state.get("airlines");
        if (als != null) {
            ((XMLable)als).handleXMLCallback(name, attrs, isBeginElement, state);
        }
    }

    public void saveAsXML(PrintWriter writer) {
        writer.write("<FlightPricingQuery>\n");
        ((XMLable)getHeader()).saveAsXML(writer);
        ((XMLable)getItinerary()).saveAsXML(writer);
        ((XMLable)getPassengers()).saveAsXML(writer);
        ((XMLable)getFareClasses()).saveAsXML(writer);
        ((XMLable)getAirlines()).saveAsXML(writer);
        writer.write("</FlightPricingQuery>\n");
    }

}