package edu.gatech.cercs.soa.core.fpq.impl;

import java.io.PrintWriter;
import java.util.Map;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpq.IHeader;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>HeaderImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class HeaderImpl implements IHeader, XMLable {

    private String customerId;
    private String queryId;
    private String queryMode;
    private int expiration; //milliseconds
    private int searchId;
    private long timestamp;
    
    public HeaderImpl() {
        queryMode = QUERYMODE_ASYNC;
        timestamp = System.currentTimeMillis();
    }
    
    public String getCustomerId() {
        return customerId;
    }

    public int getExpiration() {
        return expiration;
    }

    public String getQueryId() {
        return queryId;
    }

    public String getQueryMode() {
        return queryMode;
    }

    public int getSearchId() {
        return searchId;
    }

    public long getTimestamp() {
        return timestamp;
    }

    public void setCustomerId(String id) {
        this.customerId = id;
    }

    public void setExpiration(int exp) {
        this.expiration = exp;
    }

    public void setQueryId(String id) {
        this.queryId = id;
    }

    public void setQueryMode(String mode) {
        queryMode = QUERYMODE_POLL.equalsIgnoreCase(mode) ? 
                QUERYMODE_POLL : QUERYMODE_ASYNC;
    }

    public void setSearchId(int id) {
        this.searchId = id;
    }

    public void setTimestamp(long ts) {
        this.timestamp = ts;
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
        }
        else {
            //Closing Tag
            String str = attrs.getValue("characters");
            if (name.equalsIgnoreCase("customerid")) {
                setCustomerId(str);
            }
            else if (name.equalsIgnoreCase("queryid")) {
                setQueryId(str);
            }
            else if (name.equalsIgnoreCase("querymode")) {
                setQueryMode(str);
            }
            else if (name.equalsIgnoreCase("searchid")) {
                setSearchId(Integer.parseInt(str));
            }
            else if (name.equalsIgnoreCase("searchtimestamp")) {
                setTimestamp(Long.parseLong(str));
            }
            else if (name.equalsIgnoreCase("expiration")) {
                setExpiration(Integer.parseInt(str));
            }
        }
    }

    public void saveAsXML(PrintWriter writer) {
        StringBuffer sb = new StringBuffer();
        sb.append("<Header>\n");
        sb.append("<CustomerId>" + getCustomerId() + "</CustomerId>\n");
        sb.append("<QueryId>" + getQueryId() + "</QueryId>\n");
        sb.append("<QueryMode>" + getQueryMode() + "</QueryMode>\n");
        sb.append("<SearchId>" + getSearchId() + "</SearchId>\n");
        sb.append("<SearchTimeStamp>" + getTimestamp() + "</SearchTimeStamp>\n");
        sb.append("<Expiration>" + getExpiration() + "</Expiration>\n");
        sb.append("</Header>\n");
        writer.write(sb.toString());
    }

}
