package edu.gatech.cercs.soa.core.fpr.impl;

import java.io.PrintWriter;
import java.util.Map;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>HeaderResultImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class HeaderResultImpl implements IHeaderResult, XMLable {

    private String customerId;
    private String queryId;
    private String queryMode;
    private int expiration; //milliseconds
    private int searchId;
    private long timestamp;
    private String status;
    private String statusDetail;
    
    public HeaderResultImpl() {
        queryMode = QUERYMODE_ASYNC;
        timestamp = System.currentTimeMillis();
        status = STATUS_NONE;
    }
    
    public String getCustomerId() {
        return customerId;
    }

    public int getExpiration() {
        return expiration;
    }

    public String getQueryId() {
        return queryId;
    }

    public String getQueryMode() {
        return queryMode;
    }

    public int getSearchId() {
        return searchId;
    }
    
    public String getStatus() {
        return status;
    }
    
    public String getStatusDetail() {
        return statusDetail;
    }

    public long getTimestamp() {
        return timestamp;
    }

    public void setCustomerId(String id) {
        this.customerId = id;
    }

    public void setExpiration(int exp) {
        this.expiration = exp;
    }

    public void setQueryId(String id) {
        this.queryId = id;
    }

    public void setQueryMode(String mode) {
        queryMode = QUERYMODE_POLL.equalsIgnoreCase(mode) ? 
                QUERYMODE_POLL : QUERYMODE_ASYNC;
    }

    public void setSearchId(int id) {
        this.searchId = id;
    }

    public void setStatus(String status) {
        if (STATUS_QUEUE.equalsIgnoreCase(status)) {
            this.status = STATUS_QUEUE;
        }
        else if (STATUS_COMPLETE.equalsIgnoreCase(status)) {
            this.status = STATUS_COMPLETE;
        }
        else if (STATUS_ERROR.equalsIgnoreCase(status)) {
            this.status = STATUS_ERROR;
        }
        else {
            this.status = STATUS_NONE;
        }
    }
    
    public void setStatusDetail(String detail) {
        this.statusDetail = detail;
    }
    
    public void setTimestamp(long ts) {
        this.timestamp = ts;
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
        }
        else {
            //Closing Tag
            String str = attrs.getValue("characters");
            if (name.equalsIgnoreCase("customerid")) {
                setCustomerId(str);
            }
            else if (name.equalsIgnoreCase("queryid")) {
                setQueryId(str);
            }
            else if (name.equalsIgnoreCase("querymode")) {
                setQueryMode(str);
            }
            else if (name.equalsIgnoreCase("searchid")) {
                setSearchId(Integer.parseInt(str));
            }
            else if (name.equalsIgnoreCase("searchtimestamp")) {
                setTimestamp(Long.parseLong(str));
            }
            else if (name.equalsIgnoreCase("expiration")) {
                setExpiration(Integer.parseInt(str));
            }
            else if (name.equalsIgnoreCase("status")) {
                setStatus(str);
            }
            else if (name.equalsIgnoreCase("statusdetail")) {
                setStatusDetail(str);
            }
        }
    }

    public void saveAsXML(PrintWriter writer) {
        StringBuffer sb = new StringBuffer();
        sb.append("<Header>\n");
        sb.append("<CustomerId>" + getCustomerId() + "</CustomerId>\n");
        sb.append("<QueryId>" + getQueryId() + "</QueryId>\n");
        sb.append("<QueryMode>" + getQueryMode() + "</QueryMode>\n");
        sb.append("<SearchId>" + getSearchId() + "</SearchId>\n");
        sb.append("<SearchTimeStamp>" + getTimestamp() + "</SearchTimeStamp>\n");
        sb.append("<Expiration>" + getExpiration() + "</Expiration>\n");
        sb.append("<Status>" + getStatus() + "</Status>\n");
        sb.append("<StatusDetail>" + getStatusDetail() + "</StatusDetail>\n");
        sb.append("</Header>\n");
        writer.write(sb.toString());
    }

}
