package edu.gatech.cercs.soa.gds.ejb;


import java.util.Hashtable;

import javax.ejb.*;
import javax.jms.*;
import javax.naming.*;

import edu.gatech.cercs.soa.core.PropertyLoader;

/**
 * 
 * @ejb.bean name = "FlightPricingQueryMDB" display-name =
 *           "FlightPricingQueryMDB" description = "Flight Pricing Query Message
 *           Driven Bean" destination-type = "javax.jms.Queue"
 * 
 */
public class FlightPricingQueryMDB implements MessageDrivenBean,
        MessageListener {

    private static final long serialVersionUID = 1L; 
    
    private static final String NAMING_FACTORY = "java.naming.factory.initial";
    private static final String NAMING_PROVIDER = "java.naming.provider.url";
    private static final String CONNECTION_FACTORY_NAMES = "connectionFactoryNames";
    private static final String QUEUE_NAME = "queue.FlightPricingQueryProcessingQueue";

    private static final String PROPERTIES_FILE = "gds.properties";

    /** The MessageDrivenContext */
    //private MessageDrivenContext context;

    /**
     * An <code>ejbCreate</code> method as required by the EJB specification.
     * <br>
     * 
     * The container calls the instances <code>ejbCreate</code> method
     * immediately after instantiation.
     * 
     * @ejb.create-method
     */
    public void ejbCreate() {
    }

    /**
     * The <code>ejbRemove()</code> method as required by the EJB
     * specification.<br>
     * 
     * A container invokes this method before it ends the life of the object.
     * This happens as a result of a client's invoking a remove operation, or
     * when a container decides to terminate the object after a timeout. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *             Thrown if the instance could not perform the function
     *             requested by the container because of an system-level error.
     */
    public void ejbRemove() throws EJBException {
    }

    /**
     * Set the associated context. The container calls this method after the
     * instance creation. <br>
     * 
     * The enterprise bean instance should store the reference to the context
     * object in an instance variable. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @param newContext
     *            A MessageDrivenContext interface for the instance.
     * 
     * @throws EJBException
     *             Thrown by the method to indicate a failure caused by a
     *             system-level error.
     */
    public void setMessageDrivenContext(MessageDrivenContext newContext)
            throws EJBException {
        //context = newContext;
    }

    /**
     * Process a message. <br>
     * 
     * @param message
     *            The message to process.
     */
    public void onMessage(Message message) {
        TextMessage textMessage = (TextMessage) message;
        try {
            String query = textMessage.getText();
            // System.out.println("Query Recieved \n" + request);

            send(query);
        }
        catch (JMSException e) {
            e.printStackTrace();
        }
    }

    public void send(String query) {
        QueueConnection conn = null;
        QueueSession session = null;
        MessageProducer producer = null;
        TextMessage txtMessage = null;

        try {
            PropertyLoader propLoader = PropertyLoader.getInstance(PROPERTIES_FILE);

            Hashtable<Object, Object> env = new Hashtable<Object, Object>();
            env.put(NAMING_FACTORY, propLoader.getValue(NAMING_FACTORY));
            env.put(NAMING_PROVIDER, propLoader.getValue(NAMING_PROVIDER));
            env.put(CONNECTION_FACTORY_NAMES, propLoader
                    .getValue(CONNECTION_FACTORY_NAMES));
            env.put(QUEUE_NAME, propLoader.getValue(QUEUE_NAME));

            Context ctx = new InitialContext(env);
            QueueConnectionFactory factory = (QueueConnectionFactory) ctx
                    .lookup(propLoader.getValue(CONNECTION_FACTORY_NAMES));
            conn = factory.createQueueConnection();

            Queue myQueue = (Queue) ctx.lookup(propLoader.getValue(QUEUE_NAME));
            session = conn.createQueueSession(false,
                    javax.jms.Session.AUTO_ACKNOWLEDGE);
            producer = session.createProducer(myQueue);
            
            conn.start();
            txtMessage = session.createTextMessage();
            txtMessage.setText(query);
            producer.send(txtMessage);

            // System.out.println("Query posted to " +
            // propLoader.getValue(QUEUE_NAME) + " !");
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
        catch (JMSException e) {
            e.printStackTrace();
        }
        finally {
            try {
                if (producer != null)
                    producer.close();
                if (session != null)
                    session.close();
                if (conn != null)
                    conn.close();
            }
            catch (JMSException e) {
                e.printStackTrace();
            }
        }
    }
}
