package edu.gatech.cercs.soa.qps;

import javax.jms.*;

import org.activemq.ActiveMQConnectionFactory;

import edu.gatech.cercs.soa.core.PropertyLoader;


/**
 * Describe <code>Main</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class Main {
   
   private static final String NAMING_PROVIDER = "java.naming.provider.url";
   private static final String QUERYPROC_QUEUE_NAME = 
      "queue.FlightPricingQueryProcessingQueue";
   private static final String RESULTPROC_QUEUE_NAME = 
      "queue.FlightPricingResultProcessingQueue";
   
   private static final String PROPERTIES_FILE = "qps.properties";
   
   private PropertyLoader propLoader;
   
   public Main() {
      this.propLoader = PropertyLoader.getInstance(PROPERTIES_FILE);
   }
   
   public void recv() {
      MessageConsumer consumer = null;
      QueueConnection connection = null;
      QueueSession session = null;
      
      try {
         ActiveMQConnectionFactory connectionFactory = 
            new ActiveMQConnectionFactory(propLoader.getValue(NAMING_PROVIDER));
         connection = (QueueConnection)connectionFactory.createConnection();
         connection.start();
         
         session = connection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
         Queue queue = session.createQueue(propLoader.getValue(QUERYPROC_QUEUE_NAME));
         consumer = session.createConsumer(queue);
         
         while (true) {
            System.out.println("Waiting on " + QUERYPROC_QUEUE_NAME + "...");
            Message message = consumer.receive();
            send(connection, session, processMessage(message));
         }
      } 
      catch (Exception ex) {
         ex.printStackTrace();
      }
      finally {
         try {
            if (consumer != null) {
               consumer.close();
            }
            if (session != null) {
               session.close();
            }
            if (connection!= null) {
               connection.close();
            }
         } 
         catch (JMSException e) {
            e.printStackTrace();
         }
      }
   }
   
   private String processMessage(Message msg) {
       String result = ""; 
       try {
           TextMessage request = (TextMessage) msg;
           System.out.println("\nFlight Pricing Query: \n" + 
                   request.getText());
           QueryProcessor qp = new QueryProcessor();
           result = qp.process(request.getText());
           System.out.println("\nFlight Pricing Result: \n" +
                   result);
       } 
       catch (Exception ex) {
           ex.printStackTrace(); //TODO - log the error?
       }
   
       return result;
   }   
   
   private void send(QueueConnection connection, QueueSession session, 
            String content) {
      
      MessageProducer producer = null;
      TextMessage flightPricingResult = null;
      
      try {
         Queue queue = session.createQueue(propLoader.getValue(RESULTPROC_QUEUE_NAME));
         session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
         producer = session.createProducer(queue);

         flightPricingResult = session.createTextMessage();
         flightPricingResult.setText(content);
         
         producer.send(flightPricingResult);
         System.out.println("Result sent to " + RESULTPROC_QUEUE_NAME + "!");
      } 
      catch (JMSException e) {
         e.printStackTrace();
      }
      finally {
         try {
            if (producer != null) {
               producer.close();
            }
         } 
         catch (JMSException e) {
            e.printStackTrace();
         }
      }
   }
         
   public static void main(String[] args)  {
      Main qpsMain = new Main();
      qpsMain.recv();
   }

}

